@@ -9,9 +9,10 @@ use syn::ext::IdentExt;
 use syn::parse::{Error as ParseError, Parse, ParseStream, Result as ParseResult};
 use syn::punctuated::Punctuated;
 use syn::spanned::Spanned;
+use syn::token::Comma;
 use syn::{
     Expr, Generics, Ident, ImplItemMethod, ItemEnum, ItemFn, ItemImpl, ItemMod, ItemStruct, LitInt,
-    LitStr, PatType, Token,
+    LitStr, PatType, Token, TypePath,
 };
 
 pub mod codegen;
@@ -29,6 +30,7 @@ pub struct Program {
     pub ixs: Vec<Ix>,
     pub name: Ident,
     pub program_mod: ItemMod,
+    pub fallback_fn: Option<FallbackFn>,
 }
 
 impl Parse for Program {
@@ -91,6 +93,11 @@ pub struct IxArg {
     pub raw_arg: PatType,
 }
 
+#[derive(Debug)]
+pub struct FallbackFn {
+    raw_method: ItemFn,
+}
+
 #[derive(Debug)]
 pub struct AccountsStruct {
     // Name of the accounts struct.
@@ -99,6 +106,8 @@ pub struct AccountsStruct {
     pub generics: Generics,
     // Fields on the accounts struct.
     pub fields: Vec<AccountField>,
+    // Instruction data api expression.
+    instruction_api: Option<Punctuated<Expr, Comma>>,
 }
 
 impl Parse for AccountsStruct {
@@ -121,17 +130,23 @@ impl ToTokens for AccountsStruct {
 }
 
 impl AccountsStruct {
-    pub fn new(strct: ItemStruct, fields: Vec<AccountField>) -> Self {
+    pub fn new(
+        strct: ItemStruct,
+        fields: Vec<AccountField>,
+        instruction_api: Option<Punctuated<Expr, Comma>>,
+    ) -> Self {
         let ident = strct.ident.clone();
         let generics = strct.generics;
         Self {
             ident,
             generics,
             fields,
+            instruction_api,
         }
     }
 }
 
+#[allow(clippy::large_enum_variant)]
 #[derive(Debug)]
 pub enum AccountField {
     Field(Field),
@@ -142,6 +157,7 @@ pub enum AccountField {
 pub struct Field {
     pub ident: Ident,
     pub constraints: ConstraintGroup,
+    pub instruction_constraints: ConstraintGroup,
     pub ty: Ty,
 }
 
@@ -149,6 +165,7 @@ pub struct Field {
 pub struct CompositeField {
     pub ident: Ident,
     pub constraints: ConstraintGroup,
+    pub instruction_constraints: ConstraintGroup,
     pub symbol: String,
     pub raw_field: syn::Field,
 }
@@ -181,30 +198,30 @@ pub enum SysvarTy {
 
 #[derive(Debug, PartialEq)]
 pub struct ProgramStateTy {
-    pub account_ident: Ident,
+    pub account_type_path: TypePath,
 }
 
 #[derive(Debug, PartialEq)]
 pub struct CpiStateTy {
-    pub account_ident: Ident,
+    pub account_type_path: TypePath,
 }
 
 #[derive(Debug, PartialEq)]
 pub struct ProgramAccountTy {
     // The struct type of the account.
-    pub account_ident: Ident,
+    pub account_type_path: TypePath,
 }
 
 #[derive(Debug, PartialEq)]
 pub struct CpiAccountTy {
     // The struct type of the account.
-    pub account_ident: Ident,
+    pub account_type_path: TypePath,
 }
 
 #[derive(Debug, PartialEq)]
 pub struct LoaderTy {
     // The struct type of the account.
-    pub account_ident: Ident,
+    pub account_type_path: TypePath,
 }
 
 #[derive(Debug)]
@@ -250,14 +267,15 @@ pub struct ConstraintGroup {
     signer: Option<ConstraintSigner>,
     owner: Option<ConstraintOwner>,
     rent_exempt: Option<ConstraintRentExempt>,
-    seeds: Option<ConstraintSeeds>,
+    seeds: Option<ConstraintSeedsGroup>,
     executable: Option<ConstraintExecutable>,
     state: Option<ConstraintState>,
     associated: Option<ConstraintAssociatedGroup>,
-    belongs_to: Vec<ConstraintBelongsTo>,
+    has_one: Vec<ConstraintHasOne>,
     literal: Vec<ConstraintLiteral>,
     raw: Vec<ConstraintRaw>,
     close: Option<ConstraintClose>,
+    address: Option<ConstraintAddress>,
 }
 
 impl ConstraintGroup {
@@ -281,30 +299,33 @@ impl ConstraintGroup {
 // A single account constraint *after* merging all tokens into a well formed
 // constraint. Some constraints like "associated" are defined by multiple
 // tokens, so a merging phase is required.
+#[allow(clippy::large_enum_variant)]
 #[derive(Debug)]
 pub enum Constraint {
     Init(ConstraintInit),
     Mut(ConstraintMut),
     Signer(ConstraintSigner),
-    BelongsTo(ConstraintBelongsTo),
+    HasOne(ConstraintHasOne),
     Literal(ConstraintLiteral),
     Raw(ConstraintRaw),
     Owner(ConstraintOwner),
     RentExempt(ConstraintRentExempt),
-    Seeds(ConstraintSeeds),
+    Seeds(ConstraintSeedsGroup),
     Executable(ConstraintExecutable),
     State(ConstraintState),
     AssociatedGroup(ConstraintAssociatedGroup),
     Close(ConstraintClose),
+    Address(ConstraintAddress),
 }
 
 // Constraint token is a single keyword in a `#[account(<TOKEN>)]` attribute.
+#[allow(clippy::large_enum_variant)]
 #[derive(Debug)]
 pub enum ConstraintToken {
     Init(Context<ConstraintInit>),
     Mut(Context<ConstraintMut>),
     Signer(Context<ConstraintSigner>),
-    BelongsTo(Context<ConstraintBelongsTo>),
+    HasOne(Context<ConstraintHasOne>),
     Literal(Context<ConstraintLiteral>),
     Raw(Context<ConstraintRaw>),
     Owner(Context<ConstraintOwner>),
@@ -317,6 +338,10 @@ pub enum ConstraintToken {
     AssociatedPayer(Context<ConstraintAssociatedPayer>),
     AssociatedSpace(Context<ConstraintAssociatedSpace>),
     AssociatedWith(Context<ConstraintAssociatedWith>),
+    Address(Context<ConstraintAddress>),
+    TokenMint(Context<ConstraintTokenMint>),
+    TokenAuthority(Context<ConstraintTokenAuthority>),
+    Bump(Context<ConstraintTokenBump>),
 }
 
 impl Parse for ConstraintToken {
@@ -335,8 +360,8 @@ pub struct ConstraintMut {}
 pub struct ConstraintSigner {}
 
 #[derive(Debug, Clone)]
-pub struct ConstraintBelongsTo {
-    pub join_target: Ident,
+pub struct ConstraintHasOne {
+    pub join_target: Expr,
 }
 
 #[derive(Debug, Clone)]
@@ -351,7 +376,12 @@ pub struct ConstraintRaw {
 
 #[derive(Debug, Clone)]
 pub struct ConstraintOwner {
-    pub owner_target: Ident,
+    pub owner_target: Expr,
+}
+
+#[derive(Debug, Clone)]
+pub struct ConstraintAddress {
+    pub address: Expr,
 }
 
 #[derive(Debug, Clone)]
@@ -360,6 +390,16 @@ pub enum ConstraintRentExempt {
     Skip,
 }
 
+#[derive(Debug, Clone)]
+pub struct ConstraintSeedsGroup {
+    pub is_init: bool,
+    pub seeds: Punctuated<Expr, Token![,]>,
+    pub payer: Option<Ident>,
+    pub space: Option<Expr>,
+    pub kind: PdaKind,
+    pub bump: Option<Expr>,
+}
+
 #[derive(Debug, Clone)]
 pub struct ConstraintSeeds {
     pub seeds: Punctuated<Expr, Token![,]>,
@@ -376,30 +416,38 @@ pub struct ConstraintState {
 #[derive(Debug, Clone)]
 pub struct ConstraintAssociatedGroup {
     pub is_init: bool,
-    pub associated_target: Ident,
-    pub associated_seeds: Vec<Ident>,
+    pub associated_target: Expr,
+    pub associated_seeds: Vec<Expr>,
     pub payer: Option<Ident>,
-    pub space: Option<LitInt>,
+    pub space: Option<Expr>,
+    pub kind: PdaKind,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct ConstraintAssociated {
-    pub target: Ident,
+    pub target: Expr,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct ConstraintAssociatedPayer {
     pub target: Ident,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct ConstraintAssociatedWith {
-    pub target: Ident,
+    pub target: Expr,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct ConstraintAssociatedSpace {
-    pub space: LitInt,
+    pub space: Expr,
+}
+
+#[derive(Debug, Clone)]
+#[allow(clippy::large_enum_variant)]
+pub enum PdaKind {
+    Program { owner: Option<Expr> },
+    Token { owner: Expr, mint: Expr },
 }
 
 #[derive(Debug, Clone)]
@@ -407,6 +455,21 @@ pub struct ConstraintClose {
     pub sol_dest: Ident,
 }
 
+#[derive(Debug, Clone)]
+pub struct ConstraintTokenMint {
+    mint: Expr,
+}
+
+#[derive(Debug, Clone)]
+pub struct ConstraintTokenAuthority {
+    auth: Expr,
+}
+
+#[derive(Debug, Clone)]
+pub struct ConstraintTokenBump {
+    bump: Expr,
+}
+
 // Syntaxt context object for preserving metadata about the inner item.
 #[derive(Debug, Clone)]
 pub struct Context<T> {
